import { Component, Inject, Input, Output, EventEmitter } from '@angular/core';
import { AccordionComponent } from './accordion.component';
/**
 * Instead of using `header` attribute on the `hi-accordion-item`, you can use
 * an `hi-accordion-header``hi-accordion-body` attribute on `any` element inside of a group that
 * will be used as group's header template.
 */
var AccordionItemComponent = /** @class */ (function () {
    function AccordionItemComponent(accordion) {
        /** Emits when the opened state changes */
        this.isOpenChange = new EventEmitter();
        this._isOpen = false;
        this.accordion = accordion;
    }
    Object.defineProperty(AccordionItemComponent.prototype, "isOpen", {
        get: /** Is accordion item open or closed. This property supports two-way binding */
        function () {
            return this._isOpen;
        },
        set: function (value) {
            var _this = this;
            if (value !== this.isOpen) {
                if (value) {
                    this.accordion.closeOtherPanels(this);
                }
                this._isOpen = value;
                Promise.resolve(null).then(function () {
                    _this.isOpenChange.emit(value);
                });
            }
        },
        enumerable: true,
        configurable: true
    });
    AccordionItemComponent.prototype.ngOnInit = function () {
        this.accordion.addGroup(this);
    };
    AccordionItemComponent.prototype.ngOnDestroy = function () {
        this.accordion.removeGroup(this);
    };
    AccordionItemComponent.prototype.toggleOpen = function (event) {
        if (!this.isDisabled) {
            this.isOpen = !this.isOpen;
        }
    };
    AccordionItemComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-accordion-item',
                    template: "\n    <div class=\"card\"><div class=\"card-header\" role=\"tab\" (click)=\"toggleOpen($event)\"><div role=\"button\" class=\"accordion-toggle\"><div *ngIf=\"header\" [ngClass]=\"{'text-muted': isDisabled}\">{{ header }}</div><ng-content select=\"[hi-accordion-header]\"></ng-content></div></div><div class=\"collapse\" role=\"tabpanel\" [ngClass]=\"{show: isOpen}\"><div class=\"card-block card-body\"><ng-content></ng-content><ng-content select=\"[hi-accordion-body]\"></ng-content></div></div></div>\n  ",
                    exportAs: 'accordionItem'
                },] },
    ];
    /** @nocollapse */
    AccordionItemComponent.ctorParameters = function () { return [
        { type: AccordionComponent, decorators: [{ type: Inject, args: [AccordionComponent,] },] },
    ]; };
    AccordionItemComponent.propDecorators = {
        "header": [{ type: Input },],
        "isDisabled": [{ type: Input },],
        "isOpenChange": [{ type: Output },],
        "isOpen": [{ type: Input },],
    };
    return AccordionItemComponent;
}());
export { AccordionItemComponent };
