import { Component, Input, TemplateRef, forwardRef, ViewChild, ChangeDetectorRef } from '@angular/core';
import { AutoCompleteDirective } from './auto-complete.directive';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { HiNGConfig } from '../hi.config';
var MutipleAutoCompleteComponent = /** @class */ (function () {
    function MutipleAutoCompleteComponent(hiNGConfig, changeDetectorRef) {
        this.hiNGConfig = hiNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.appendBody = false;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.delay = hiNGConfig.autoComplete.delay;
        this.minLength = hiNGConfig.autoComplete.minLength;
        this.itemTemplate = hiNGConfig.autoComplete.itemTemplate;
        this.noResultItemTemplate = hiNGConfig.autoComplete.noResultItemTemplate;
        this.formatter = hiNGConfig.autoComplete.formatter;
        this.valueParser = hiNGConfig.autoComplete.valueParser;
        this.id = MutipleAutoCompleteComponent.ID_SEED++;
    }
    MutipleAutoCompleteComponent.prototype.writeValue = function (obj) {
        this.mutipleItems = obj || [];
        this.changeDetectorRef.markForCheck();
    };
    MutipleAutoCompleteComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    MutipleAutoCompleteComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    MutipleAutoCompleteComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    MutipleAutoCompleteComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.autoCompleteDirective) {
            this.autoCompleteDirective.registerOnTouched(function () { return _this.onTouched(); });
        }
    };
    MutipleAutoCompleteComponent.prototype.toggle = function ($event) {
        if (this.autoCompleteDirective) {
            this.autoCompleteDirective.toggle($event);
        }
    };
    MutipleAutoCompleteComponent.prototype.onMutipleSearch = function () {
        this.onTouched();
        if (this.mutipleItems.indexOf(this.selectValue) === -1) {
            this.mutipleItems.push(this.selectValue);
            this.onChange(this.mutipleItems);
        }
        this.selectValue = null;
    };
    MutipleAutoCompleteComponent.prototype.removeLabel = function (label) {
        if (this.disabled) {
            return;
        }
        this.onTouched();
        if (this.mutipleItems.indexOf(label) !== -1) {
            this.mutipleItems = this.mutipleItems.filter(function (item) { return item !== label; });
            this.onChange(this.mutipleItems);
        }
    };
    MutipleAutoCompleteComponent.prototype.onMutipleSearchBoxBackspace = function ($event) {
        var _this = this;
        this.onTouched();
        if (!$event.target.value && this.mutipleItems.length) {
            this.mutipleItems.pop();
            this.onChange(this.mutipleItems);
            setTimeout(function () { return _this.autoCompleteDirective.positionPopup(); }, 0);
        }
    };
    MutipleAutoCompleteComponent.ID_SEED = 0;
    MutipleAutoCompleteComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-mutiple-auto-complete',
                    template: "\n    <label for=\"mutiple-label-auto-complete-{{id}}\" class=\"mutiple-label-auto-complete {{cssClass}}\"><ul><li *ngFor=\"let item of mutipleItems\"><span class=\"label label-info\" [ngClass]=\"{disabled: disabled}\"><i aria-hidden=\"true\" class=\"label-close\" (click)=\"removeLabel(item)\">&times;</i>{{formatter(item)}}</span></li><li *ngIf=\"!disabled\"><input id=\"mutiple-label-auto-complete-{{id}}\" name=\"mutipleAutoComplete\" class=\"auto-complete-control\" [(ngModel)]=\"selectValue\" (selectValueChange)=\"onMutipleSearch()\" [onSearch]=\"onSearch\" [cssClass]=\"popupCssClass\" [delay]=\"delay\" [minLength]=\"minLength\" [itemTemplate]=\"itemTemplate\" [noResultItemTemplate]=\"noResultItemTemplate\" [formatter]=\"formatter\" [valueParser]=\"valueParser\" (keydown.backspace)=\"onMutipleSearchBoxBackspace($event)\" [disabled]=\"disabled\" [dataSource]=\"dataSource\" [appendBody]=\"appendBody\" reAutoComplete></li></ul></label>\n  ",
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return MutipleAutoCompleteComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    MutipleAutoCompleteComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    MutipleAutoCompleteComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "popupCssClass": [{ type: Input },],
        "delay": [{ type: Input },],
        "minLength": [{ type: Input },],
        "appendBody": [{ type: Input },],
        "dataSource": [{ type: Input },],
        "itemTemplate": [{ type: Input },],
        "noResultItemTemplate": [{ type: Input },],
        "formatter": [{ type: Input },],
        "valueParser": [{ type: Input },],
        "onSearch": [{ type: Input },],
        "autoCompleteDirective": [{ type: ViewChild, args: [AutoCompleteDirective,] },],
    };
    return MutipleAutoCompleteComponent;
}());
export { MutipleAutoCompleteComponent };
