import { Component, ChangeDetectionStrategy, Input, ChangeDetectorRef, forwardRef, Output, EventEmitter } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var CheckboxGroupComponent = /** @class */ (function () {
    function CheckboxGroupComponent(hiNGConfig, changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.disabled = false;
        this.valueChange = new EventEmitter();
        this.value = [];
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.formatter = hiNGConfig.checkboxGroup.formatter;
        this.valueParser = hiNGConfig.checkboxGroup.valueParser;
    }
    CheckboxGroupComponent.prototype.writeValue = function (obj) {
        this.value = obj instanceof Array ? obj : (obj ? [obj] : []);
        this.changeDetectorRef.markForCheck();
    };
    CheckboxGroupComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    CheckboxGroupComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    CheckboxGroupComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    CheckboxGroupComponent.prototype.selectAll = function () {
        var _this = this;
        this.value = this.options.map(function (item) { return _this.valueParser(item); });
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.unselectAll = function () {
        this.value = [];
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.invertSelect = function () {
        var _this = this;
        var current = this.value || [];
        this.value = this.options
            .filter(function (item) { return current.indexOf(_this.valueParser(item)) === -1; })
            .map(function (item) { return _this.valueParser(item); });
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.onCheckBoxChange = function (item, checkbox) {
        this.onTouched();
        this.value = this.value || [];
        var value = this.valueParser(item);
        if (checkbox.checked) {
            this.value = this.value.concat([value]);
        }
        else {
            this.value = this.value.filter(function (valueItem) { return valueItem !== value; });
        }
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.isChecked = function (item) {
        return this.value.indexOf(this.valueParser(item)) !== -1;
    };
    CheckboxGroupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-checkbox-group',
                    template: "\n    <div [ngClass]=\"{'checkbox-inline': inline, 'checkbox': !inline}\" *ngFor=\"let item of options\"><label><input #checkbox type=\"checkbox\" [checked]=\"isChecked(item)\" [disabled]=\"disabled\" (change)=\"onCheckBoxChange(item, checkbox)\">{{formatter(item)}}</label></div>\n  ",
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'checkboxGroup',
                    host: { '[class]': 'cssClass' },
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return CheckboxGroupComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    CheckboxGroupComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    CheckboxGroupComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "options": [{ type: Input },],
        "inline": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "formatter": [{ type: Input },],
        "valueParser": [{ type: Input },],
        "valueChange": [{ type: Output },],
    };
    return CheckboxGroupComponent;
}());
export { CheckboxGroupComponent };
