import { Component, Input, Optional } from '@angular/core';
import { DataTable } from './data-table.directive';
import * as _ from 'lodash';
var BootstrapPaginator = /** @class */ (function () {
    function BootstrapPaginator() {
        this.rowsOnPageSet = [];
        this.minRowsOnPage = 0;
    }
    BootstrapPaginator.prototype.ngOnChanges = function (changes) {
        if (changes.rowsOnPageSet) {
            this.minRowsOnPage = _.min(this.rowsOnPageSet);
        }
    };
    BootstrapPaginator.decorators = [
        { type: Component, args: [{
                    selector: 'hi-data-pagination',
                    template: "\n    <hi-data-paginator #p [table]=\"table\">\n      <ul class=\"pagination\" *ngIf=\"p.dataLength > p.rowsOnPage\">\n        <li class=\"page-item\" [class.disabled]=\"p.activePage <= 1\" (click)=\"p.setPage(1)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">&laquo;</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage > 4 && p.activePage + 1 > p.lastPage\" (click)=\"p.setPage(p.activePage - 4)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage - 4}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage > 3 && p.activePage + 2 > p.lastPage\" (click)=\"p.setPage(p.activePage - 3)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage - 3}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage > 2\" (click)=\"p.setPage(p.activePage - 2)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage - 2}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage > 1\" (click)=\"p.setPage(p.activePage - 1)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage - 1}}</a>\n        </li>\n        <li class=\"page-item active\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage + 1 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 1)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage + 1}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage + 2 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 2)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage + 2}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage + 3 <= p.lastPage && p.activePage < 3\" (click)=\"p.setPage(p.activePage + 3)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage + 3}}</a>\n        </li>\n        <li class=\"page-item\" *ngIf=\"p.activePage + 4 <= p.lastPage && p.activePage < 2\" (click)=\"p.setPage(p.activePage + 4)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage + 4}}</a>\n        </li>\n        <li class=\"page-item\" [class.disabled]=\"p.activePage >= p.lastPage\" (click)=\"p.setPage(p.lastPage)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">&raquo;</a>\n        </li>\n      </ul>\n      <ul class=\"pagination pull-right float-sm-right\" *ngIf=\"p.dataLength > minRowsOnPage\">\n        <li class=\"page-item\" *ngFor=\"let rows of rowsOnPageSet\" [class.active]=\"p.rowsOnPage===rows\" (click)=\"p.setRowsOnPage(rows)\">\n          <a class=\"page-link\" style=\"cursor: pointer\">{{rows}}</a>\n        </li>\n      </ul>\n    </hi-data-paginator>\n  "
                },] },
    ];
    /** @nocollapse */
    BootstrapPaginator.propDecorators = {
        "rowsOnPageSet": [{ type: Input, args: ['rowsOnPageSet',] },],
        "table": [{ type: Input, args: ['table',] },],
    };
    return BootstrapPaginator;
}());
export { BootstrapPaginator };
var Paginator = /** @class */ (function () {
    function Paginator(injectMfTable) {
        var _this = this;
        this.injectMfTable = injectMfTable;
        this.onPageChangeSubscriber = function (event) {
            _this.activePage = event.activePage;
            _this.rowsOnPage = event.rowsOnPage;
            _this.dataLength = event.dataLength;
            _this.lastPage = Math.ceil(_this.dataLength / _this.rowsOnPage);
        };
    }
    Paginator.prototype.ngOnChanges = function (changes) {
        this.table = this.inputMfTable || this.injectMfTable;
        this.onPageChangeSubscriber(this.table.getPage());
        this.table.onPageChange.subscribe(this.onPageChangeSubscriber);
    };
    Paginator.prototype.setPage = function (pageNumber) {
        this.table.setPage(pageNumber, this.rowsOnPage);
    };
    Paginator.prototype.setRowsOnPage = function (rowsOnPage) {
        this.table.setPage(this.activePage, rowsOnPage);
    };
    Paginator.decorators = [
        { type: Component, args: [{
                    selector: 'hi-data-paginator',
                    template: "\n    <ng-content></ng-content>"
                },] },
    ];
    /** @nocollapse */
    Paginator.ctorParameters = function () { return [
        { type: DataTable, decorators: [{ type: Optional },] },
    ]; };
    Paginator.propDecorators = {
        "inputMfTable": [{ type: Input, args: ['table',] },],
    };
    return Paginator;
}());
export { Paginator };
var DataSort = /** @class */ (function () {
    function DataSort(table) {
        this.table = table;
        this.isSortedByMeAsc = false;
        this.isSortedByMeDesc = false;
    }
    DataSort.prototype.ngOnInit = function () {
        var _this = this;
        this.table.onSortChange.subscribe(function (event) {
            _this.isSortedByMeAsc = (event.sortBy == _this.sortBy && event.sortOrder == 'asc');
            _this.isSortedByMeDesc = (event.sortBy == _this.sortBy && event.sortOrder == 'desc');
        });
    };
    DataSort.prototype.sort = function () {
        if (this.isSortedByMeAsc) {
            this.table.setSort(this.sortBy, 'desc');
        }
        else {
            this.table.setSort(this.sortBy, 'asc');
        }
    };
    DataSort.decorators = [
        { type: Component, args: [{
                    selector: 'hi-data-sort',
                    template: "\n    <a style=\"cursor: pointer\" (click)=\"sort()\" class=\"text-nowrap\">\n      <ng-content></ng-content>\n      <span *ngIf=\"isSortedByMeAsc\" class=\"glyphicon glyphicon-triangle-top\" aria-hidden=\"true\"></span>\n      <span *ngIf=\"isSortedByMeDesc\" class=\"glyphicon glyphicon-triangle-bottom\" aria-hidden=\"true\"></span>\n    </a>"
                },] },
    ];
    /** @nocollapse */
    DataSort.ctorParameters = function () { return [
        { type: DataTable, },
    ]; };
    DataSort.propDecorators = {
        "sortBy": [{ type: Input, args: ['by',] },],
    };
    return DataSort;
}());
export { DataSort };
