import { HostListener, EventEmitter, Output, Input, Directive, ContentChild, ElementRef } from '@angular/core';
var DraggableHandleDirective = /** @class */ (function () {
    function DraggableHandleDirective(elementRef) {
        this.elementRef = elementRef;
    }
    DraggableHandleDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[reDraggableHandle]'
                },] },
    ];
    /** @nocollapse */
    DraggableHandleDirective.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    return DraggableHandleDirective;
}());
export { DraggableHandleDirective };
var DraggableDirective = /** @class */ (function () {
    function DraggableDirective() {
        this.disabled = false;
        // https://developer.mozilla.org/zh-CN/docs/Web/API/DataTransfer/effectAllowed
        this.dragEffect = 'copy';
        this.onDragStart = new EventEmitter();
        this.onDragEnd = new EventEmitter();
        this.onDragging = new EventEmitter();
    }
    DraggableDirective.prototype.dragStart = function ($event) {
        if (!this.draggableHandle || this.draggableHandle.elementRef.nativeElement === this.handle) {
            $event.stopPropagation();
            $event.dataTransfer.effectAllowed = this.dragEffect;
            $event.dataTransfer.setData(DraggableDirective.DRAGGABLE_DATA_KEY, JSON.stringify({
                group: this.group,
                data: this.dragData
            }));
            this.onDragStart.emit($event);
            return;
        }
        $event.preventDefault();
    };
    DraggableDirective.prototype.drag = function ($event) {
        this.onDragging.emit($event);
    };
    DraggableDirective.prototype.dragEnd = function ($event) {
        this.onDragEnd.emit($event);
    };
    DraggableDirective.prototype.mouseover = function (event) {
        this.handle = event.target;
    };
    DraggableDirective.prototype.mouseleave = function (event) {
        this.handle = null;
    };
    DraggableDirective.DRAGGABLE_DATA_KEY = 'text';
    DraggableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[reDraggable]',
                    host: {
                        '[draggable]': '!disabled'
                    }
                },] },
    ];
    /** @nocollapse */
    DraggableDirective.propDecorators = {
        "disabled": [{ type: Input },],
        "group": [{ type: Input, args: ['reDraggable',] },],
        "dragData": [{ type: Input },],
        "dragEffect": [{ type: Input },],
        "onDragStart": [{ type: Output },],
        "onDragEnd": [{ type: Output },],
        "onDragging": [{ type: Output },],
        "draggableHandle": [{ type: ContentChild, args: [DraggableHandleDirective,] },],
        "dragStart": [{ type: HostListener, args: ['dragstart', ['$event'],] },],
        "drag": [{ type: HostListener, args: ['drag', ['$event'],] },],
        "dragEnd": [{ type: HostListener, args: ['dragend', ['$event'],] },],
        "mouseover": [{ type: HostListener, args: ['mouseover', ['$event'],] },],
        "mouseleave": [{ type: HostListener, args: ['mouseleave', ['$event'],] },],
    };
    return DraggableDirective;
}());
export { DraggableDirective };
