import { HostListener, EventEmitter, Output, Input, Directive } from '@angular/core';
import { DraggableDirective } from './draggable.directive';
var DroppableDirective = /** @class */ (function () {
    function DroppableDirective() {
        this.dropEffect = 'move';
        this.onDragEnter = new EventEmitter();
        this.onDragLeave = new EventEmitter();
        this.onDrop = new EventEmitter();
        this.onDragOver = new EventEmitter();
    }
    DroppableDirective.prototype.drop = function ($event) {
        if ((!this.group || this.isDropGroup($event)) && (!this.acceptDrop || this.acceptDrop($event))) {
            this.startDrop($event);
        }
    };
    DroppableDirective.prototype.dragEnter = function ($event) {
        $event.preventDefault();
        if (this.dropEffect) {
            $event.dataTransfer.dropEffect = this.dropEffect;
        }
        this.onDragEnter.emit($event);
    };
    DroppableDirective.prototype.dragLeave = function ($event) {
        $event.preventDefault();
        this.onDragLeave.emit($event);
    };
    DroppableDirective.prototype.dragOver = function ($event) {
        $event.preventDefault();
        this.onDragOver.emit($event);
    };
    DroppableDirective.prototype.isDropGroup = function ($event) {
        var contextData = $event.dataTransfer.getData(DraggableDirective.DRAGGABLE_DATA_KEY);
        if (contextData) {
            var draggableData = JSON.parse(contextData);
            return this.group === draggableData.group;
        }
    };
    DroppableDirective.prototype.startDrop = function ($event) {
        $event.preventDefault();
        $event.stopPropagation();
        this.onDrop.emit($event);
    };
    DroppableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[reDroppable]',
                },] },
    ];
    /** @nocollapse */
    DroppableDirective.propDecorators = {
        "group": [{ type: Input, args: ['reDroppable',] },],
        "acceptDrop": [{ type: Input },],
        "dropEffect": [{ type: Input },],
        "onDragEnter": [{ type: Output },],
        "onDragLeave": [{ type: Output },],
        "onDrop": [{ type: Output },],
        "onDragOver": [{ type: Output },],
        "drop": [{ type: HostListener, args: ['drop', ['$event'],] },],
        "dragEnter": [{ type: HostListener, args: ['dragenter', ['$event'],] },],
        "dragLeave": [{ type: HostListener, args: ['dragleave', ['$event'],] },],
        "dragOver": [{ type: HostListener, args: ['dragover', ['$event'],] },],
    };
    return DroppableDirective;
}());
export { DroppableDirective };
