import { Component, ContentChildren, ElementRef, HostBinding, Input, QueryList, TemplateRef } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { trimWhiteSpace } from '../utils/trim-whitespace';
import { InputDirective } from './input.directive';
var InputGroupComponent = /** @class */ (function () {
    function InputGroupComponent(el) {
        this.el = el;
        this._addOnBefore = '';
        this._addOnAfter = '';
        this._prefix = '';
        this._suffix = '';
        this._size = 'default';
        this._compact = false;
        this._search = false;
    }
    Object.defineProperty(InputGroupComponent.prototype, "hiSize", {
        get: function () {
            return this._size;
        },
        set: function (value) {
            this._size = value;
            this.updateChildrenInputSize();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiCompact", {
        get: function () {
            return this._compact;
        },
        set: function (value) {
            this._compact = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiAddOnBefore", {
        get: function () {
            return this._addOnBefore;
        },
        set: function (value) {
            this.isAddOnBeforeString = !(value instanceof TemplateRef);
            this._addOnBefore = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiAddOnAfter", {
        get: function () {
            return this._addOnAfter;
        },
        set: function (value) {
            this.isAddOnAfterString = !(value instanceof TemplateRef);
            this._addOnAfter = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiPrefix", {
        get: function () {
            return this._prefix;
        },
        set: function (value) {
            this.isPrefixString = !(value instanceof TemplateRef);
            this._prefix = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiSuffix", {
        get: function () {
            return this._suffix;
        },
        set: function (value) {
            this.isSuffixString = !(value instanceof TemplateRef);
            this._suffix = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "hiSearch", {
        get: function () {
            return this._search;
        },
        set: function (value) {
            this._search = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isLarge", {
        get: function () {
            return this.hiSize === 'large';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isSmall", {
        get: function () {
            return this.hiSize === 'small';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isAffix", {
        get: function () {
            return !!(this.hiSuffix || this.hiPrefix || this.hiPrefixIcon || this.hiSuffixIcon);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isAddOn", {
        get: function () {
            return !!(this.hiAddOnAfter || this.hiAddOnBefore || this.hiAddOnAfterIcon || this.hiAddOnBeforeIcon);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isGroup", {
        get: function () {
            return (!this.isAffix) && (!this.isAddOn);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isLargeGroup", {
        get: function () {
            return this.isGroup && this.isLarge;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isLargeGroupWrapper", {
        get: function () {
            return this.isAddOn && this.isLarge;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isLargeAffix", {
        get: function () {
            return this.isAffix && this.isLarge;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isLargeSearch", {
        get: function () {
            return this.hiSearch && this.isLarge;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isSmallGroup", {
        get: function () {
            return this.isGroup && this.isSmall;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isSmallAffix", {
        get: function () {
            return this.isAffix && this.isSmall;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isSmallGroupWrapper", {
        get: function () {
            return this.isAddOn && this.isSmall;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputGroupComponent.prototype, "isSmallSearch", {
        get: function () {
            return this.hiSearch && this.isSmall;
        },
        enumerable: true,
        configurable: true
    });
    InputGroupComponent.prototype.updateChildrenInputSize = function () {
        var _this = this;
        if (this.InputDirectiveQueryList) {
            this.InputDirectiveQueryList.forEach(function (item) { return item.hiSize = _this.hiSize; });
        }
    };
    InputGroupComponent.prototype.ngAfterContentInit = function () {
        this.updateChildrenInputSize();
    };
    InputGroupComponent.prototype.ngAfterViewInit = function () {
        trimWhiteSpace(this.el.nativeElement);
    };
    InputGroupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-input-group',
                    preserveWhitespaces: false,
                    template: "\n    <span class=\"hi-input-wrapper hi-input-group\" *ngIf=\"isAddOn\">\n      <span class=\"hi-input-group-addon\" *ngIf=\"hiAddOnBefore || hiAddOnBeforeIcon\">\n        <i [ngClass]=\"hiAddOnBeforeIcon\" *ngIf=\"hiAddOnBeforeIcon\"></i>\n        <ng-container *ngIf=\"isAddOnBeforeString; else addOnBeforeTemplate\">{{ hiAddOnBefore }}</ng-container>\n        <ng-template #addOnBeforeTemplate>\n          <ng-template [ngTemplateOutlet]=\"hiAddOnBefore\"></ng-template>\n        </ng-template>\n      </span>\n      <ng-template *ngTemplateOutlet=\"contentTemplate\"></ng-template>\n      <span class=\"hi-input-group-addon\" *ngIf=\"hiAddOnAfter || hiAddOnAfterIcon\">\n        <i [ngClass]=\"hiAddOnAfterIcon\" *ngIf=\"hiAddOnAfterIcon\"></i>\n        <ng-container *ngIf=\"isAddOnAfterString; else addOnAfterTemplate\">{{ hiAddOnAfter }}</ng-container>\n        <ng-template #addOnAfterTemplate>\n          <ng-template [ngTemplateOutlet]=\"hiAddOnAfter\"></ng-template>\n        </ng-template>\n      </span>\n    </span>\n    <ng-template [ngIf]=\"isAffix\">\n      <span class=\"hi-input-prefix\" *ngIf=\"hiPrefix || hiPrefixIcon\">\n        <i [ngClass]=\"hiPrefixIcon\" *ngIf=\"hiPrefixIcon\"></i>\n        <ng-container *ngIf=\"isPrefixString; else prefixTemplate\">{{ hiPrefix }}</ng-container>\n        <ng-template #prefixTemplate>\n          <ng-template [ngTemplateOutlet]=\"hiPrefix\"></ng-template>\n        </ng-template>\n      </span>\n      <ng-template *ngTemplateOutlet=\"contentTemplate\"></ng-template>\n      <span class=\"hi-input-suffix\" *ngIf=\"hiSuffix || hiSuffixIcon\">\n        <i [ngClass]=\"hiSuffixIcon\" *ngIf=\"hiSuffixIcon\"></i>\n        <ng-container *ngIf=\"isSuffixString; else suffixTemplate\">{{ hiSuffix }}</ng-container>\n        <ng-template #suffixTemplate>\n          <ng-template [ngTemplateOutlet]=\"hiSuffix\"></ng-template>\n        </ng-template>\n      </span>\n    </ng-template>\n    <ng-template [ngIf]=\"isGroup\" *ngTemplateOutlet=\"contentTemplate\"></ng-template>\n    <ng-template #contentTemplate>\n      <ng-content></ng-content>\n    </ng-template>\n  ",
                },] },
    ];
    /** @nocollapse */
    InputGroupComponent.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    InputGroupComponent.propDecorators = {
        "InputDirectiveQueryList": [{ type: ContentChildren, args: [InputDirective,] },],
        "hiAddOnBeforeIcon": [{ type: Input },],
        "hiAddOnAfterIcon": [{ type: Input },],
        "hiPrefixIcon": [{ type: Input },],
        "hiSuffixIcon": [{ type: Input },],
        "hiSize": [{ type: Input },],
        "hiCompact": [{ type: Input }, { type: HostBinding, args: ["class.hi-input-group-compact",] },],
        "hiAddOnBefore": [{ type: Input },],
        "hiAddOnAfter": [{ type: Input },],
        "hiPrefix": [{ type: Input },],
        "hiSuffix": [{ type: Input },],
        "hiSearch": [{ type: Input }, { type: HostBinding, args: ["class.hi-input-search-enter-button",] }, { type: HostBinding, args: ["class.hi-input-search",] },],
        "isAffix": [{ type: HostBinding, args: ['class.hi-input-affix-wrapper',] },],
        "isAddOn": [{ type: HostBinding, args: ['class.hi-input-group-wrapper',] },],
        "isGroup": [{ type: HostBinding, args: ['class.hi-input-group',] },],
        "isLargeGroup": [{ type: HostBinding, args: ["class.hi-input-group-lg",] },],
        "isLargeGroupWrapper": [{ type: HostBinding, args: ["class.hi-input-group-wrapper-lg",] },],
        "isLargeAffix": [{ type: HostBinding, args: ["class.hi-input-affix-wrapper-lg",] },],
        "isLargeSearch": [{ type: HostBinding, args: ["class.hi-input-search-lg",] },],
        "isSmallGroup": [{ type: HostBinding, args: ["class.hi-input-group-sm",] },],
        "isSmallAffix": [{ type: HostBinding, args: ["class.hi-input-affix-wrapper-sm",] },],
        "isSmallGroupWrapper": [{ type: HostBinding, args: ["class.hi-input-group-wrapper-sm",] },],
        "isSmallSearch": [{ type: HostBinding, args: ["class.hi-input-search-sm",] },],
    };
    return InputGroupComponent;
}());
export { InputGroupComponent };
