import { Injectable, Injector, ComponentFactoryResolver } from '@angular/core';
import { ModalComponent } from './modal.component';
import { HiNGConfig } from '../hi.config';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/catch';
import { _throw } from 'rxjs/observable/throw';
var ModalService = /** @class */ (function () {
    function ModalService(hiNGConfig, resolver, injector) {
        this.hiNGConfig = hiNGConfig;
        this.resolver = resolver;
        this.injector = injector;
        this.instances = [];
    }
    ModalService.prototype.open = function (options) {
        var _this = this;
        var rootContainer = options.rootContainer || this.hiNGConfig.rootContainer;
        if (!rootContainer) {
            throw new Error('Should setup ViewContainerRef on modal options or hi config service!');
        }
        if (options.animation === undefined || options.animation === null) {
            options.animation = this.hiNGConfig.modal.animation;
        }
        var componentFactory = this.resolver.resolveComponentFactory(ModalComponent);
        var injector = options.injector || this.injector;
        var modalRef = rootContainer
            .createComponent(componentFactory, rootContainer.length, injector);
        this.instances.push(modalRef);
        var instance = modalRef.instance;
        var dismissResult = instance.addContent(options, this.instances.length)
            .do(function () { return _this.close(modalRef); })
            .catch(function (error) {
            _this.close(modalRef);
            return _throw(error);
        });
        instance.open();
        return dismissResult;
    };
    ModalService.prototype.closeAll = function () {
        var _this = this;
        this.instances.forEach(function (modalRef) { return _this.close(modalRef); });
    };
    ModalService.prototype.close = function (modalRef) {
        var _this = this;
        this.instances.splice(this.instances.indexOf(modalRef), 1);
        var subscriber = modalRef.instance.close()
            .subscribe(function (_) {
            subscriber.unsubscribe();
            if (!_this.instances.length) {
                modalRef.instance.cleanup();
            }
            modalRef.destroy();
        });
    };
    ModalService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ModalService.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ComponentFactoryResolver, },
        { type: Injector, },
    ]; };
    return ModalService;
}());
export { ModalService };
