import { Component, Input, ViewChild, ElementRef, Renderer2, HostListener } from '@angular/core';
import { WindowRef } from '../window-ref/window-ref.service';
import { OverlayContentComponent } from './overlay-content.component';
import { centerWindowPosition } from '../utils/dom-utils';
var OverlayComponent = /** @class */ (function () {
    function OverlayComponent(windowRef, renderer) {
        this.windowRef = windowRef;
        this.renderer = renderer;
    }
    OverlayComponent.prototype.addContent = function (options) {
        this.overlayOptions = options;
        if (!this.overlayOptions.html && this.overlayOptions.component) {
            this.overlayContent.addContent(this.overlayOptions);
        }
        this.adjustOverlayPosition();
    };
    OverlayComponent.prototype.onWinResize = function () {
        this.adjustOverlayPosition();
    };
    OverlayComponent.prototype.adjustOverlayPosition = function () {
        var _this = this;
        setTimeout(function () {
            var pos = centerWindowPosition(_this.overlayBody, _this.windowRef);
            _this.renderer.setStyle(_this.overlayBody.nativeElement, 'top', pos.top + "px");
            _this.renderer.setStyle(_this.overlayBody.nativeElement, 'left', pos.left + "px");
        }, 0);
    };
    OverlayComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-overlay',
                    template: "\n    <div #overlayBody class=\"overlay-body\" style=\"z-index: 1050;top:300px;left: 200px\"><div *ngIf=\"overlayOptions?.html\" [innerHTML]=\"overlayOptions.html\"></div><hi-overlay-content #overlayContent style=\"display: none\"></hi-overlay-content></div><hi-modal-backdrop [isOpen]=\"true\" [modalOptions]=\"overlayOptions\"></hi-modal-backdrop>\n  "
                },] },
    ];
    /** @nocollapse */
    OverlayComponent.ctorParameters = function () { return [
        { type: WindowRef, },
        { type: Renderer2, },
    ]; };
    OverlayComponent.propDecorators = {
        "overlayOptions": [{ type: Input },],
        "overlayBody": [{ type: ViewChild, args: ['overlayBody',] },],
        "overlayContent": [{ type: ViewChild, args: [OverlayContentComponent,] },],
        "onWinResize": [{ type: HostListener, args: ['window:resize', [],] },],
    };
    return OverlayComponent;
}());
export { OverlayComponent };
