import { Component, Input, Output, EventEmitter, ChangeDetectionStrategy, TemplateRef } from '@angular/core';
import { HiNGConfig } from '../hi.config';
var PaginationComponent = /** @class */ (function () {
    function PaginationComponent(hiNGConfig) {
        this.total = 0;
        this.pageIndex = 1;
        this.pageIndexChange = new EventEmitter();
        this.showPages = [];
        this.totalPage = 0;
        this.boundary = hiNGConfig.pagination.boundary;
        this.pageSize = hiNGConfig.pagination.pageSize;
        this.maxItems = hiNGConfig.pagination.maxItems;
        this.firstLink = hiNGConfig.pagination.button.first;
        this.lastLink = hiNGConfig.pagination.button.last;
        this.preLink = hiNGConfig.pagination.button.pre;
        this.nextLink = hiNGConfig.pagination.button.next;
        this.firstLinkCssClass = hiNGConfig.pagination.button.firstLinkCssClass;
        this.preLinkCssClass = hiNGConfig.pagination.button.preLinkCssClass;
        this.nextLinkCssClass = hiNGConfig.pagination.button.nextLinkCssClass;
        this.lastLinkCssClass = hiNGConfig.pagination.button.lastLinkCssClass;
        this.size = hiNGConfig.pagination.size;
    }
    PaginationComponent.prototype.first = function () {
        if (this.pageIndex !== 1) {
            this.onPageIndexChange(1);
        }
    };
    PaginationComponent.prototype.last = function () {
        var last = Math.max(this.totalPage, 1);
        if (this.pageIndex !== last) {
            this.onPageIndexChange(last);
        }
    };
    PaginationComponent.prototype.prev = function () {
        if (this.hasPrev()) {
            this.onPageIndexChange(this.pageIndex - 1);
        }
    };
    PaginationComponent.prototype.next = function () {
        if (this.hasNext()) {
            this.onPageIndexChange(this.pageIndex + 1);
        }
    };
    PaginationComponent.prototype.preRange = function () {
        var pre = this.showPages[0] - 1;
        this.onPageIndexChange(Math.max(pre, 1));
    };
    PaginationComponent.prototype.nextRange = function () {
        var next = this.showPages[this.showPages.length - 1] + 1;
        this.onPageIndexChange(Math.min(next, this.totalPage));
    };
    PaginationComponent.prototype.onPageIndexChange = function (pageIndex) {
        if (this.pageIndex !== pageIndex) {
            this.pageIndexChange.emit(pageIndex);
        }
    };
    PaginationComponent.prototype.hasPrev = function () {
        return this.pageIndex > 1;
    };
    PaginationComponent.prototype.hasNext = function () {
        return this.pageIndex < this.totalPage;
    };
    PaginationComponent.prototype.getTotalPage = function () {
        return Math.ceil(this.total / this.pageSize);
    };
    PaginationComponent.prototype.ngOnChanges = function (changes) {
        var shouldUpdateRanges = PaginationComponent.EFFECT_PAGE_RANGE_KEYS.some(function (key) { return !!changes[key]; });
        if (shouldUpdateRanges) {
            this.totalPage = this.getTotalPage();
            this.pageIndex = Math.max(Math.min(this.pageIndex, this.totalPage), 1);
            this.updateShowPageRange();
        }
    };
    PaginationComponent.prototype.updateShowPageRange = function () {
        if (!this.totalPage) {
            this.showPages = [];
            return;
        }
        if (this.totalPage <= this.maxItems) {
            this.showPages = new Array(this.totalPage).fill(0).map(function (_, i) { return i + 1; });
            return;
        }
        this.showPages = this.extractRange();
    };
    PaginationComponent.prototype.extractRange = function () {
        var _this = this;
        var showPages = [this.pageIndex];
        var start = this.pageIndex - 1;
        var end = this.pageIndex + 1;
        var arriveLeftBound = function (index) { return index < 1; };
        var arriveRightBound = function (index) { return index > _this.totalPage; };
        var fullPageRang = function (pages) { return pages.length >= _this.maxItems; };
        while (!(fullPageRang(showPages) || (arriveLeftBound(start) && arriveRightBound(end)))) {
            if (!arriveLeftBound(start)) {
                showPages.unshift((start--));
            }
            if (!fullPageRang(showPages) && !arriveRightBound(end)) {
                showPages.push(end++);
            }
        }
        return showPages;
    };
    PaginationComponent.EFFECT_PAGE_RANGE_KEYS = ['total', 'pageSize', 'pageIndex', 'maxItems'];
    PaginationComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-pagination',
                    template: "\n    <ul class=\"pagination {{size? 'pagination-' + size: ''}}\"><li><ng-template *ngIf=\"headTemplate\" [ngTemplateOutlet]=\"headTemplate\" [ngTemplateOutletContext]=\"{ $implicit: this }\"></ng-template></li><li *ngIf=\"boundary\" [ngClass]=\"{disabled: pageIndex == 1}\"><a [attr.aria-label]=\"firstLink\" (click)=\"first()\"><span aria-hidden=\"true\" class=\"{{firstLinkCssClass}}\">{{firstLink}}</span></a></li><li [ngClass]=\"{disabled: pageIndex == 1}\"><a (click)=\"prev()\" [attr.aria-label]=\"preLink\"><span aria-hidden=\"true\" class=\"{{preLinkCssClass}}\">{{preLink}}</span></a></li><li *ngIf=\"showPages[0] > 1\" (click)=\"preRange()\"><a class=\"ellipsis\">...</a></li><li *ngFor=\"let item of showPages\" [ngClass]=\"{active: item == pageIndex}\"><a (click)=\"onPageIndexChange(item)\">{{item}} <span class=\"sr-only\" *ngIf=\"item == pageIndex\"></span></a></li><li *ngIf=\"showPages[showPages.length -1] < totalPage\" (click)=\"nextRange()\"><a class=\"ellipsis\">...</a></li><li [ngClass]=\"{disabled: pageIndex == totalPage}\"><a (click)=\"next()\" [attr.aria-label]=\"nextLink\"><span aria-hidden=\"true\" class=\"{{nextLinkCssClass}}\">{{nextLink}}</span></a></li><li *ngIf=\"boundary\" [ngClass]=\"{disabled: pageIndex == totalPage}\"><a [attr.aria-label]=\"lastLink\" (click)=\"last()\"><span aria-hidden=\"true\" class=\"{{lastLinkCssClass}}\">{{lastLink}}</span></a></li><li class=\"page-item\"><ng-template *ngIf=\"tailTemplate\" [ngTemplateOutlet]=\"tailTemplate\" [ngTemplateOutletContext]=\"{ $implicit: this }\"></ng-template></li></ul>\n  ",
                    exportAs: 'pagination',
                    changeDetection: ChangeDetectionStrategy.OnPush
                },] },
    ];
    /** @nocollapse */
    PaginationComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
    ]; };
    PaginationComponent.propDecorators = {
        "boundary": [{ type: Input },],
        "total": [{ type: Input },],
        "pageSize": [{ type: Input },],
        "pageIndex": [{ type: Input },],
        "pageIndexChange": [{ type: Output },],
        "maxItems": [{ type: Input },],
        "firstLink": [{ type: Input },],
        "lastLink": [{ type: Input },],
        "preLink": [{ type: Input },],
        "nextLink": [{ type: Input },],
        "size": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "headTemplate": [{ type: Input },],
        "tailTemplate": [{ type: Input },],
        "firstLinkCssClass": [{ type: Input },],
        "preLinkCssClass": [{ type: Input },],
        "nextLinkCssClass": [{ type: Input },],
        "lastLinkCssClass": [{ type: Input },],
    };
    return PaginationComponent;
}());
export { PaginationComponent };
