import { Component, ChangeDetectionStrategy, forwardRef, Input, ChangeDetectorRef, EventEmitter, Output } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { HiNGConfig } from '../hi.config';
var RadioGroupComponent = /** @class */ (function () {
    function RadioGroupComponent(hiNGConfig, changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.disabled = false;
        this.valueChange = new EventEmitter();
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.formatter = hiNGConfig.radioGroup.formatter;
        this.valueParser = hiNGConfig.radioGroup.valueParser;
        this.name = "hi-radio-group-" + RadioGroupComponent.NAME_SEED++;
    }
    RadioGroupComponent.prototype.writeValue = function (obj) {
        this.value = obj;
        this.changeDetectorRef.markForCheck();
    };
    RadioGroupComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    RadioGroupComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    RadioGroupComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    RadioGroupComponent.prototype.onRadioChange = function (item) {
        this.onTouched();
        this.value = this.valueParser(item);
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    RadioGroupComponent.prototype.isChecked = function (item) {
        return this.value === this.valueParser(item);
    };
    RadioGroupComponent.NAME_SEED = 0;
    RadioGroupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-radio-group',
                    template: "\n    <div [ngClass]=\"{'radio-inline': inline, 'radio': !inline}\" *ngFor=\"let item of options\"><label><input type=\"radio\" [name]=\"name\" [value]=\"valueParser(item)\" [checked]=\"isChecked(item)\" [disabled]=\"disabled\" (change)=\"onRadioChange(item)\">{{formatter(item)}}</label></div>\n  ",
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: { '[class]': 'cssClass' },
                    exportAs: 'radioGroup',
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return RadioGroupComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    RadioGroupComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    RadioGroupComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "options": [{ type: Input },],
        "inline": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "formatter": [{ type: Input },],
        "valueParser": [{ type: Input },],
        "valueChange": [{ type: Output },],
    };
    return RadioGroupComponent;
}());
export { RadioGroupComponent };
