import { Component, ChangeDetectionStrategy, forwardRef, Input, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { HiNGConfig } from '../hi.config';
var SelectButtonComponent = /** @class */ (function () {
    function SelectButtonComponent(changeDetectorRef, hiNGConfig) {
        this.changeDetectorRef = changeDetectorRef;
        this.disabled = false;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.type = hiNGConfig.selectButton.type;
        this.justified = hiNGConfig.selectButton.justified;
        this.multiple = hiNGConfig.selectButton.multiple;
    }
    SelectButtonComponent.prototype.writeValue = function (obj) {
        this.value = obj;
        this.changeDetectorRef.markForCheck();
    };
    SelectButtonComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SelectButtonComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SelectButtonComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    SelectButtonComponent.prototype.isSelected = function (item) {
        var itemValue = item.value || item.text;
        if (this.value) {
            return this.multiple ? this.value.indexOf(itemValue) !== -1 : this.value === itemValue;
        }
    };
    SelectButtonComponent.prototype.selectedItem = function (item) {
        this.onTouched();
        var itemValue = item.value || item.text;
        if (this.multiple) {
            return this.fillMultipleValue(itemValue);
        }
        this.fillSingleValue(itemValue);
    };
    SelectButtonComponent.prototype.fillSingleValue = function (itemValue) {
        if (this.value !== itemValue) {
            this.value = itemValue;
            this.onChange(this.value);
        }
    };
    SelectButtonComponent.prototype.fillMultipleValue = function (itemValue) {
        var value = this.value || [];
        if (value.indexOf(itemValue) === -1) {
            this.value = value.concat([itemValue]);
        }
        else {
            this.value = value.filter(function (item) {
                return item !== itemValue;
            });
        }
        this.onChange(this.value);
    };
    SelectButtonComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-select-button',
                    template: "\n    <div class=\"btn-group\" role=\"group\" *ngFor=\"let item of items\"><button type=\"button\" class=\"btn btn-{{type}}\" [ngClass]=\"{active: isSelected(item)}\" [disabled]=\"disabled\" (click)=\"selectedItem(item)\">{{item.text}}</button></div>\n  ",
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: {
                        '[class]': '(cssClass || "") + " btn-group " + (size ? "btn-group-" + size : "") ',
                        'role': 'group',
                        'aria-label': 'Button_group'
                    },
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return SelectButtonComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    SelectButtonComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: HiNGConfig, },
    ]; };
    SelectButtonComponent.propDecorators = {
        "size": [{ type: Input },],
        "type": [{ type: Input },],
        "justified": [{ type: Input },],
        "multiple": [{ type: Input },],
        "items": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "disabled": [{ type: Input },],
    };
    return SelectButtonComponent;
}());
export { SelectButtonComponent };
