import { Component, EventEmitter, Input, Output, QueryList, ViewChildren } from '@angular/core';
import { OptionComponent } from './option.component';
import { Subject } from 'rxjs/Subject';
import { merge } from 'rxjs/operators/merge';
import { OptionLiComponent } from './option-li.component';
import { defaultFilterOption, OptionPipe } from './option.pipe';
var OptionContainerComponent = /** @class */ (function () {
    function OptionContainerComponent() {
        this.isInit = false;
        this.isAddTagOptionDisplay = false;
        this.listOfAllTemplateOption = [];
        this.listOfTagOption = [];
        this.listOfFilterOption = [];
        // tslint:disable-next-line:no-any
        this.hiListOfSelectedValueChange = new EventEmitter();
        this.hiListOfTemplateOptionChange = new EventEmitter();
        this.hiClickOption = new EventEmitter();
        this.hiScrollToBottom = new EventEmitter();
        this.hiMode = 'default';
        this.hiServerSearch = false;
        this.hiFilterOption = defaultFilterOption;
        this.hiMaxMultipleCount = Infinity;
        // tslint:disable-next-line:no-any
        this.compareWith = function (o1, o2) { return o1 === o2; };
    }
    Object.defineProperty(OptionContainerComponent.prototype, "hiSearchValue", {
        get: function () {
            return this._searchValue;
        },
        set: function (value) {
            this._searchValue = value;
            this.updateAddTagOptionDisplay();
            this.updateListOfFilterOption();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionContainerComponent.prototype, "hiListOfSelectedValue", {
        // tslint:disable-next-line:no-any
        get: 
        // tslint:disable-next-line:no-any
        function () {
            return this._listOfSelectedValue;
        },
        set: 
        // tslint:disable-next-line:no-any
        function (value) {
            if (this._listOfSelectedValue !== value) {
                this._listOfSelectedValue = value;
                /** should clear activedOption when listOfSelectedValue change **/
                this.clearActivatedOption();
                this.refreshAllOptionStatus(false);
            }
        },
        enumerable: true,
        configurable: true
    });
    OptionContainerComponent.prototype.addTagOption = function () {
        if (this.hiListOfSelectedValue.length < this.hiMaxMultipleCount) {
            this.hiListOfSelectedValue = this.hiListOfSelectedValue.concat([this.hiSearchValue]);
            this.hiListOfSelectedValueChange.emit(this.hiListOfSelectedValue);
        }
    };
    OptionContainerComponent.prototype.clickOption = function (option, isPressEnter) {
        this.updateSelectedOption(option, isPressEnter);
        this.hiClickOption.emit();
    };
    OptionContainerComponent.prototype.onKeyDownUl = function (e) {
        var _this = this;
        if ([38, 40, 13].indexOf(e.keyCode) > -1) {
            e.preventDefault();
            var activeIndex = this.listOfFilterOption.findIndex(function (item) { return item === _this.activatedOption; });
            if (e.keyCode === 38) {
                // arrow up
                var preIndex = activeIndex > 0 ? (activeIndex - 1) : (this.listOfFilterOption.length - 1);
                this.setActiveOption(this.listOfFilterOption[preIndex]);
            }
            else if (e.keyCode === 40) {
                // arrow down
                var nextIndex = activeIndex < this.listOfFilterOption.length - 1 ? (activeIndex + 1) : 0;
                this.setActiveOption(this.listOfFilterOption[nextIndex]);
            }
            else if (e.keyCode === 13) {
                // enter
                if (this.isTagsMode) {
                    if (!this.isAddTagOptionDisplay) {
                        this.clickOption(this.activatedOption, true);
                    }
                    else {
                        this.addTagOption();
                        this.hiClickOption.emit();
                    }
                }
                else {
                    this.clickOption(this.activatedOption, true);
                }
            }
        }
    };
    OptionContainerComponent.prototype.resetActiveOption = function () {
        var _this = this;
        var firstActiveOption = this.listOfAllTemplateOption.concat(this.listOfTagOption).find(function (item) { return _this.compareWith(item.hiValue, _this.hiListOfSelectedValue[0]); });
        this.setActiveOption(firstActiveOption);
    };
    OptionContainerComponent.prototype.clearActivatedOption = function () {
        this.setActiveOption(null);
    };
    OptionContainerComponent.prototype.setActiveOption = function (option, scroll) {
        if (scroll === void 0) { scroll = true; }
        this.activatedOption = option;
        if (scroll) {
            this.scrollIntoView();
        }
    };
    OptionContainerComponent.prototype.scrollIntoView = function () {
        var _this = this;
        if (this.listOfHiOptionLiComponent && this.listOfHiOptionLiComponent.length) {
            var targetLi_1 = this.listOfHiOptionLiComponent.find(function (o) { return o.hiOption === _this.activatedOption; });
            if (targetLi_1 && targetLi_1.el) {
                setTimeout(function () { return targetLi_1.el.scrollIntoView(false); }, 150);
            }
        }
    };
    OptionContainerComponent.prototype.updateSelectedOption = function (option, isPressEnter) {
        var _this = this;
        /** update listOfSelectedOption -> update hiListOfSelectedValue -> emit hiListOfSelectedValueChange **/
        if (option && !option.hiDisabled) {
            var changed = false;
            this.setActiveOption(option);
            var listOfSelectedValue = this.hiListOfSelectedValue.slice();
            if (this.isMultipleOrTags) {
                var targetValue = listOfSelectedValue.find(function (o) { return _this.compareWith(o, option.hiValue); });
                if (targetValue) {
                    if (!isPressEnter) {
                        /** should not toggle option when press enter **/
                        listOfSelectedValue.splice(listOfSelectedValue.indexOf(targetValue), 1);
                        changed = true;
                    }
                }
                else if (this.hiListOfSelectedValue.length < this.hiMaxMultipleCount) {
                    listOfSelectedValue.push(option.hiValue);
                    changed = true;
                }
            }
            else if (!this.compareWith(listOfSelectedValue[0], option.hiValue)) {
                listOfSelectedValue = [option.hiValue];
                changed = true;
            }
            /** update selectedValues when click option **/
            if (changed) {
                this._listOfSelectedValue = listOfSelectedValue;
                this.hiListOfSelectedValueChange.emit(this.hiListOfSelectedValue);
                if (this.isTagsMode) {
                    this.refreshAllOptionStatus(false);
                }
            }
        }
    };
    OptionContainerComponent.prototype.refreshListOfTagOption = function () {
        var _this = this;
        if (this.isTagsMode) {
            /** refresh tags option **/
            var listOfTagsOption_1 = [];
            this.hiListOfSelectedValue.forEach(function (value) {
                var existedOption = _this.listOfAllTemplateOption.find(function (o) { return _this.compareWith(o.hiValue, value); });
                if (!existedOption) {
                    var hiOptionComponent = new OptionComponent();
                    hiOptionComponent.hiValue = value;
                    hiOptionComponent.hiLabel = value;
                    listOfTagsOption_1.push(hiOptionComponent);
                }
            });
            this.listOfTagOption = listOfTagsOption_1;
        }
    };
    OptionContainerComponent.prototype.refreshListOfAllTemplateOption = function () {
        var _this = this;
        this.listOfAllTemplateOption = this.listOfHiOptionComponent.toArray().concat(this.listOfHiOptionGroupComponent.toArray().reduce(function (pre, cur) { return pre.concat(cur.listOfHiOptionComponent.toArray()); }, []));
        Promise.resolve().then(function () { return _this.hiListOfTemplateOptionChange.emit(_this.listOfAllTemplateOption); });
    };
    OptionContainerComponent.prototype.refreshAllOptionStatus = function (isTemplateOptionChange) {
        /** update hiListOfSelectedValue | update option list -> update listOfAllTemplateOption -> update listOfSelectedOption -> update activatedOption **/
        if (this.isInit) {
            if (isTemplateOptionChange) {
                this.refreshListOfAllTemplateOption();
            }
            this.refreshListOfTagOption();
            this.updateListOfFilterOption();
            this.updateAddTagOptionDisplay();
        }
    };
    OptionContainerComponent.prototype.updateListOfFilterOption = function () {
        this.listOfFilterOption = new OptionPipe().transform(this.listOfAllTemplateOption.concat(this.listOfTagOption), this.hiSearchValue, this.hiFilterOption, this.hiServerSearch);
        if (this.hiSearchValue) {
            this.setActiveOption(this.listOfFilterOption[0]);
        }
    };
    /** watch options change in option group **/
    /** watch options change in option group **/
    OptionContainerComponent.prototype.watchSubOptionChanges = /** watch options change in option group **/
    function () {
        var _this = this;
        this.unsubscribeOption();
        var optionChanges$ = new Subject().asObservable().pipe(merge(this.listOfHiOptionGroupComponent.changes)).pipe(merge(this.listOfHiOptionComponent.changes));
        if (this.listOfHiOptionGroupComponent.length) {
            this.listOfHiOptionGroupComponent.forEach(function (group) { return optionChanges$ = group.listOfHiOptionComponent ? optionChanges$.pipe(merge(group.listOfHiOptionComponent.changes)) : optionChanges$; });
        }
        this.optionSubscription = optionChanges$.subscribe(function () { return _this.refreshAllOptionStatus(true); });
    };
    OptionContainerComponent.prototype.unsubscribeGroup = function () {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
            this.groupSubscription = null;
        }
    };
    OptionContainerComponent.prototype.unsubscribeOption = function () {
        if (this.optionSubscription) {
            this.optionSubscription.unsubscribe();
            this.optionSubscription = null;
        }
    };
    Object.defineProperty(OptionContainerComponent.prototype, "isTagsMode", {
        get: function () {
            return this.hiMode === 'tags';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionContainerComponent.prototype, "isMultipleOrTags", {
        get: function () {
            return this.hiMode === 'tags' || this.hiMode === 'multiple';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionContainerComponent.prototype, "isNotFoundDisplay", {
        get: function () {
            return (!this.isTagsMode) && (!this.listOfFilterOption.length);
        },
        enumerable: true,
        configurable: true
    });
    OptionContainerComponent.prototype.updateAddTagOptionDisplay = function () {
        var listOfAllOption = this.listOfAllTemplateOption.concat(this.listOfTagOption).map(function (item) { return item.hiLabel; });
        var isMatch = listOfAllOption.indexOf(this.hiSearchValue) > -1;
        this.isAddTagOptionDisplay = this.isTagsMode && this.hiSearchValue && (!isMatch);
    };
    OptionContainerComponent.prototype.dropDownScroll = function (e, ul) {
        e.preventDefault();
        e.stopPropagation();
        if (ul && (ul.scrollHeight - ul.scrollTop === ul.clientHeight)) {
            this.hiScrollToBottom.emit();
        }
    };
    OptionContainerComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        this.isInit = true;
        this.refreshAllOptionStatus(true);
        this.watchSubOptionChanges();
        this.groupSubscription = this.listOfHiOptionGroupComponent.changes.subscribe(function () { return _this.watchSubOptionChanges(); });
    };
    OptionContainerComponent.prototype.ngOnDestroy = function () {
        this.unsubscribeGroup();
        this.unsubscribeOption();
    };
    OptionContainerComponent.decorators = [
        { type: Component, args: [{
                    selector: '[hi-option-container]',
                    preserveWhitespaces: false,
                    template: "\n    <ul\n      #dropdownUl\n      class=\"hi-select-dropdown-menu hi-select-dropdown-menu-root hi-select-dropdown-menu-vertical\"\n      role=\"menu\"\n      (keydown)=\"onKeyDownUl($event)\"\n      (scroll)=\"dropDownScroll($event,dropdownUl)\"\n      tabindex=\"0\">\n      <li\n        *ngIf=\"isNotFoundDisplay\"\n        hi-select-unselectable\n        class=\"hi-select-dropdown-menu-item hi-select-dropdown-menu-item-disabled\">\n        {{ hiNotFoundContent ? hiNotFoundContent : ('Select.notFoundContent' | hiI18n) }}\n      </li>\n      <li\n        *ngIf=\"isAddTagOptionDisplay\"\n        hi-select-unselectable\n        (click)=\"addTagOption()\"\n        class=\"hi-select-dropdown-menu-item hi-select-dropdown-menu-item-active\">\n        {{ hiSearchValue }}\n      </li>\n      <li\n        hi-option-li\n        [compareWith]=\"compareWith\"\n        *ngFor=\"let option of listOfHiOptionComponent | hiFilterOptionPipe : hiSearchValue : hiFilterOption : hiServerSearch \"\n        (click)=\"clickOption(option,false)\"\n        [hiActiveOption]=\"activatedOption\"\n        [hiOption]=\"option\"\n        [hiListOfSelectedValue]=\"hiListOfSelectedValue\">\n      </li>\n      <li\n        *ngFor=\"let group of listOfHiOptionGroupComponent | hiSubFilterOptionPipe : hiSearchValue : hiFilterOption : hiServerSearch\"\n        class=\"hi-select-dropdown-menu-item-group\">\n        <div\n          class=\"hi-select-dropdown-menu-item-group-title\"\n          [attr.title]=\"group.isLabelString ? group.hiLabel : ''\">\n          <ng-container *ngIf=\"group.isLabelString; else labelTemplate\">{{ group.hiLabel }}</ng-container>\n          <ng-template #labelTemplate>\n            <ng-template [ngTemplateOutlet]=\"group.hiLabel\"></ng-template>\n          </ng-template>\n        </div>\n        <ul class=\"hi-select-dropdown-menu-item-group-list\">\n          <li\n            hi-option-li\n            [compareWith]=\"compareWith\"\n            *ngFor=\"let option of group.listOfHiOptionComponent | hiFilterOptionPipe : hiSearchValue : hiFilterOption : hiServerSearch\"\n            (click)=\"clickOption(option,false)\"\n            [hiActiveOption]=\"activatedOption\"\n            [hiShowActive]=\"!isAddTagOptionDisplay\"\n            [hiOption]=\"option\"\n            [hiListOfSelectedValue]=\"hiListOfSelectedValue\">\n          </li>\n        </ul>\n      </li>\n      <li\n        hi-option-li\n        [compareWith]=\"compareWith\"\n        *ngFor=\"let option of listOfTagOption | hiFilterOptionPipe : hiSearchValue : hiFilterOption : hiServerSearch \"\n        (click)=\"clickOption(option,false)\"\n        [hiActiveOption]=\"activatedOption\"\n        [hiShowActive]=\"!isAddTagOptionDisplay\"\n        [hiOption]=\"option\"\n        [hiListOfSelectedValue]=\"hiListOfSelectedValue\">\n      </li>\n    </ul>"
                },] },
    ];
    /** @nocollapse */
    OptionContainerComponent.propDecorators = {
        "listOfHiOptionLiComponent": [{ type: ViewChildren, args: [OptionLiComponent,] },],
        "listOfHiOptionComponent": [{ type: Input },],
        "listOfHiOptionGroupComponent": [{ type: Input },],
        "hiListOfSelectedValueChange": [{ type: Output },],
        "hiListOfTemplateOptionChange": [{ type: Output },],
        "hiClickOption": [{ type: Output },],
        "hiScrollToBottom": [{ type: Output },],
        "hiMode": [{ type: Input },],
        "hiServerSearch": [{ type: Input },],
        "hiFilterOption": [{ type: Input },],
        "hiMaxMultipleCount": [{ type: Input },],
        "hiNotFoundContent": [{ type: Input },],
        "compareWith": [{ type: Input },],
        "hiSearchValue": [{ type: Input },],
        "hiListOfSelectedValue": [{ type: Input },],
    };
    return OptionContainerComponent;
}());
export { OptionContainerComponent };
