import { Component, Input, ChangeDetectionStrategy, forwardRef, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { HiNGConfig } from '../hi.config';
var SwitchComponent = /** @class */ (function () {
    function SwitchComponent(hiNGConfig, changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.onText = hiNGConfig.switchBtn.onText;
        this.offText = hiNGConfig.switchBtn.offText;
        this.type = hiNGConfig.switchBtn.type;
    }
    SwitchComponent.prototype.toggle = function () {
        if (this.disabled) {
            return;
        }
        this.onTouched();
        this.checked = !this.checked;
        this.onChange(this.checked);
    };
    SwitchComponent.prototype.writeValue = function (obj) {
        this.checked = obj;
        this.changeDetectorRef.markForCheck();
    };
    SwitchComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SwitchComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SwitchComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    SwitchComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-switch',
                    template: "\n    <div class=\"switch {{cssClass}} {{size ? 'btn-' + size : ''}}\" (click)=\"toggle()\" [ngClass]=\"{'disabled': disabled}\"><span class=\"switch-left {{checked ? 'bg-'+ type + ' active' : ''}}\">{{onText}} </span><span class=\"switch-right {{!checked ? 'bg-'+ type + ' active' : ''}}\">{{offText}}</span></div>\n  ",
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return SwitchComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    SwitchComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    SwitchComponent.propDecorators = {
        "type": [{ type: Input },],
        "size": [{ type: Input },],
        "disabled": [{ type: Input },],
        "onText": [{ type: Input },],
        "offText": [{ type: Input },],
        "cssClass": [{ type: Input },],
    };
    return SwitchComponent;
}());
export { SwitchComponent };
