import { Component, Input, Output, EventEmitter, TemplateRef } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { TreeViewService } from './tree-view.service';
var TreeViewComponent = /** @class */ (function () {
    function TreeViewComponent(hiNGConfig, treeViewService) {
        this.treeViewService = treeViewService;
        this.checkable = false;
        this.lazyLoad = false;
        this.allowDraggable = false;
        this.allowMutipleSelected = false;
        this.nodeItemClicked = new EventEmitter();
        this.nodeItemDroped = new EventEmitter();
        this.nodeItemDbClicked = new EventEmitter();
        this.nodeItemCheckedChanged = new EventEmitter();
        this.nodeItemExpended = new EventEmitter();
        this.valueField = hiNGConfig.treeView.valueField;
        this.textField = hiNGConfig.treeView.textField;
        this.leafIcon = hiNGConfig.treeView.leafIcon;
        this.expendIcon = hiNGConfig.treeView.expendIcon;
        this.collapseIcon = hiNGConfig.treeView.collapseIcon;
        this.loadingIcon = hiNGConfig.treeView.loadingIcon;
    }
    TreeViewComponent.prototype.onNodeItemExpended = function (node) {
        this.nodeItemExpended.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemClicked = function (node) {
        if (!this.allowMutipleSelected) {
            if (this.selectNode && this.selectNode !== node) {
                this.selectNode.$select = false;
            }
            this.selectNode = node;
        }
        this.nodeItemClicked.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemDbClicked = function (node) {
        this.nodeItemDbClicked.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemCheckedChanged = function (node) {
        this.nodeItemCheckedChanged.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemDroped = function ($event) {
        var _this = this;
        var target = $event.target;
        if (!target) {
            return;
        }
        var source = $event.data.data.node;
        var sourceParent = $event.data.data.parent;
        if (sourceParent && sourceParent[this.valueField]) {
            var matchNode = this.getTreeNodeByValue(sourceParent[this.valueField]);
            if (matchNode) {
                matchNode.node.children = matchNode.node.children
                    .filter(function (nodeItem) { return nodeItem[_this.valueField] !== source[_this.valueField]; });
            }
        }
        else {
            if (this.treeData.length < 2) {
                return;
            }
            this.treeData = this.treeData.filter(function (nodeItem) { return nodeItem[_this.valueField] !== source[_this.valueField]; });
        }
        target.children = target.children || [];
        source.$select = false;
        target.children.push(source);
        this.nodeItemDroped.emit($event);
    };
    TreeViewComponent.prototype.getSelectNodes = function () {
        return this.treeViewService.getSelectNodes(this.treeData);
    };
    TreeViewComponent.prototype.getCheckedNodes = function () {
        return this.treeViewService.getCheckedNodes(this.treeData);
    };
    TreeViewComponent.prototype.getMatchedItems = function (match) {
        return this.treeViewService.getMatchedItems(this.treeData, match);
    };
    TreeViewComponent.prototype.getFirstMatchedItem = function (match) {
        return this.treeViewService.getFirstMatchedItem(this.treeData, match);
    };
    TreeViewComponent.prototype.getTreeNodeByValue = function (value) {
        return this.treeViewService.getTreeNodeByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.expendAllNodes = function () {
        return this.treeViewService.expendAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.collapseAllNodes = function () {
        return this.treeViewService.collapseAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.expendNodesByLevel = function (level) {
        this.treeViewService.expendNodesByLevel(this.treeData, level);
    };
    TreeViewComponent.prototype.expendNodesByValue = function (value) {
        this.treeViewService.expendNodesByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.getNodePathByValue = function (value) {
        return this.treeViewService.getNodePathByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.checkAllNodes = function () {
        return this.treeViewService.checkAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.unCheckAllNodes = function () {
        return this.treeViewService.unCheckAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.appendNodes = function (parentId, nodes) {
        return this.treeViewService.appendNodes(this.treeData, this.valueField, parentId, nodes);
    };
    TreeViewComponent.prototype.removeNode = function (value) {
        return this.treeViewService.removeNode(this.treeData, this.valueField, value);
    };
    TreeViewComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-tree-view',
                    template: "\n    <hi-tree-panel class=\"{{cssClass}}\" [treeData]=\"treeData\" [textField]=\"textField\" [valueField]=\"valueField\" [nodeCssClass]=\"nodeCssClass\" [iconField]=\"iconField\" [checkable]=\"checkable\" [lazyLoad]=\"lazyLoad\" [loadingIcon]=\"loadingIcon\" [loadChildren]=\"loadChildren\" [allowDraggable]=\"allowDraggable\" [nodeItemTemplate]=\"nodeItemTemplate\" [nodeItemToolbarTemplate]=\"nodeItemToolbarTemplate\" [leafIcon]=\"leafIcon\" [expendIcon]=\"expendIcon\" [collapseIcon]=\"collapseIcon\" [allowMutipleSelected]=\"allowMutipleSelected\" (nodeItemCheckedChanged)=\"onNodeItemCheckedChanged($event)\"></hi-tree-panel>\n  ",
                    exportAs: 'treeView'
                },] },
    ];
    /** @nocollapse */
    TreeViewComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: TreeViewService, },
    ]; };
    TreeViewComponent.propDecorators = {
        "treeData": [{ type: Input },],
        "valueField": [{ type: Input },],
        "textField": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "nodeCssClass": [{ type: Input },],
        "iconField": [{ type: Input },],
        "checkable": [{ type: Input },],
        "lazyLoad": [{ type: Input },],
        "loadingIcon": [{ type: Input },],
        "loadChildren": [{ type: Input },],
        "allowDraggable": [{ type: Input },],
        "allowMutipleSelected": [{ type: Input },],
        "nodeItemTemplate": [{ type: Input },],
        "nodeItemToolbarTemplate": [{ type: Input },],
        "leafIcon": [{ type: Input },],
        "expendIcon": [{ type: Input },],
        "collapseIcon": [{ type: Input },],
        "nodeItemClicked": [{ type: Output },],
        "nodeItemDroped": [{ type: Output },],
        "nodeItemDbClicked": [{ type: Output },],
        "nodeItemCheckedChanged": [{ type: Output },],
        "nodeItemExpended": [{ type: Output },],
    };
    return TreeViewComponent;
}());
export { TreeViewComponent };
